@@ -12,6 +12,8 @@ import threading
 import time
 from time import sleep
 
+import numpy
+
 from phootonics_controller.base_controllers.config import VCM_PORT, CAVITY1_INDEX, CAVITY2_INDEX, CAVITY3_INDEX, \
     CAVITY1_URL, CAVITY2_URL, CAVITY3_URL, ADC_CHANNEL, REF, configuration_dictionary, CAVITY4_URL, CAVITY4_INDEX, \
     EXT_VCM_PORT, get_scanning_wavelengths
@@ -67,9 +69,9 @@ class MockADCDevice:
 
 class ScanCommand:
 
-    def __init__(self, controller, wavelegths, positions):
+    def __init__(self, controller, wavelengths, positions):
         self._controller = controller
-        self.wavelengths = wavelegths
+        self.wavelengths = wavelengths
         self.positions = positions
         self.results = {'active_cavity': [],
                         'x_set_point': [],
@@ -163,6 +165,146 @@ class ScanCommand:
                 'results': self.results}
 
 
+class FastScanCommand:
+
+    def __init__(self, controller, wavelengths, amplitude_x, amplitude_y):
+        self._controller = controller
+        self.wavelengths = wavelengths
+        self.amplitude_x = amplitude_x
+        self.amplitude_y = amplitude_y
+        self.results = {'active_cavity': [],
+                        'x_set_point': [],
+                        'x': [],
+                        'y_set_point': [],
+                        'y': [],
+                        'wavelength_set_point': [],
+                        'wavelength': [],
+                        'detector_voltage': []}
+        self.stopped = threading.Event()
+        self.currentX = None
+        self.currentY = None
+        self.activeCavity = None
+        self.currentWavelength = None
+        self.detectorVoltage = None
+        self.error = None
+        self._testWithMocks = True
+        self._completedWl = 0
+        self._detectorAcquisitionThread = None
+        self._detectorAcquisitions = []
+        self._vcmAcquisitions = []
+
+    def start_detector_acquisition(self):
+        self._detectorAcquisitionThread = threading.Thread(target=self._run_acquisition)
+        self._detectorAcquisitionThread.start()
+
+    def _run_acquisition(self):
+        while True:
+            if self.stopped.is_set():
+                return
+            self._detectorAcquisitions += (time.time(), self._controller.adc.read())
+            if self.stopped.wait(0.001):
+                return
+
+    def record_callback(self, x, y, t):
+        self._vcmAcquisitions += (t, x, y)
+
+    def clear_registers(self):
+        self._detectorAcquisitions = []
+        self._vcmAcquisitions = []
+
+    def execute(self):
+        try:
+            if not self._controller.testMode or self._testWithMocks:
+                self._controller.stop_all_pulsing()
+                self._controller.move_to_cavity_1()
+                self._controller.get_active_cavity().activate_s2()
+                self.start_detector_acquisition()
+                self._controller.extVcm._device.start_x_signal_generator(self.amplitude_x, 25, 0, 0)
+                self._controller.extVcm._device.start_y_signal_generator(self.amplitude_y, 1, 0, 0)
+                self._controller.extVcm.register_record_callback(self.record_callback)
+                for wavelength in self.wavelengths:
+                    self._completedWl += 1
+                    if self.stopped.is_set():
+                        return
+                    logger.info('going to wavelength {}'.format(wavelength))
+                    cavity_index = self._controller.get_cavity_from_wavelength(wavelength)
+                    if cavity_index is None:
+                        logger.info('cannot reach wavelength {}: skip to next wavelength'.format(wavelength))
+                        continue
+                    logger.info('selecting cavity {}'.format(cavity_index))
+                    self._controller.select_cavity(cavity_index)
+                    cavity = self._controller.get_cavity_from_index(cavity_index)
+                    cavity.move_to_wavelength(wavelength)
+
+                    self.clear_registers()
+
+                    if self.stopped.wait(1):
+                        return
+
+                    detector = numpy.array(self._detectorAcquisitions)
+                    vcm = numpy.array(self._vcmAcquisitions)
+
+                    detectorTimes = detector[:, 0]
+                    detectorValues = detector[:, 1]
+                    vcmTimes = vcm[:, 0]
+                    vcmX = vcm[:, 1]
+                    vcmY = vcm[:, 2]
+
+                    self.clear_registers()
+
+                    interpX = numpy.interp(detectorTimes, vcmTimes, vcmX)
+                    interpY = numpy.interp(detectorTimes, vcmTimes, vcmY)
+
+                    activeCavity = self._controller.get_active_cavity_index()
+                    self.results['active_cavity'].append(activeCavity)
+
+                    self.results['x'].append(interpX.tolist())
+                    self.results['y'].append(interpY.tolist())
+                    self.results['wavelength_set_point'].append(wavelength)
+                    self.results['wavelength'].append(cavity.get_wavelength())
+                    self.results['detector_voltage'].append(detectorValues.tolist())
+
+                    self.activeCavity = activeCavity
+                    self.currentX = None
+                    self.currentY = None
+                    self.detectorVoltage = None
+
+
+            else:
+                if not self.stopped.wait(2.0):
+                    self.results = {'active_cavity': [1, 1, 2, 2, 3, 3, 4, 4],
+                                    'x_set_point': [0, 1, 0, 1, 0, 1, 0, 1],
+                                    'x': [0.2, 1.2, 0.2, 1.2, 0.2, 1.2, 0.2, 1.2],
+                                    'y_set_point': [0, 1, 0, 1, 0, 1, 0, 1],
+                                    'y': [0.23, 1.23, 0.23, 1.23, 0.23, 1.23, 0.23, 1.23],
+                                    'wavelength_set_point': [1300, 1301, 1302, 1303, 1304, 1305, 1306, 1307],
+                                    'wavelength': [1300.2, 1301.1, 1302.4, 1303.3, 1304.2, 1305.6, 1306.2, 1307.1],
+                                    'detector_voltage': [0.2, 0.5, 0.3, 0.11, 0.2, 0.5, 0.3, 0.11]}
+
+        except Exception as e:
+            self.error = str(e)
+            logger.exception(e)
+        finally:
+            self.stop()
+            self._controller.stop_all_pulsing()
+            self._controller.extVcm.reset_static_input()
+            self._controller.extVcm.unregister_record_callback(self.record_callback)
+
+    def stop(self):
+        self.stopped.set()
+
+    def to_dict(self):
+        return {'runtime': {'active_cavity': self.activeCavity,
+                            'current_X': self.currentX,
+                            'current_y': self.currentY,
+                            'error': self.error,
+                            'detector_voltage': self.detectorVoltage,
+                            'completed': self._completedWl/len(self.wavelengths)},
+                'results': self.results}
+
+
+
+
 class AlreadyScanningException(Exception):
     pass
 
@@ -245,11 +387,24 @@ class MainController:
                 raise AlreadyScanningException
             self._scanCommandThread = None
         self._scanCommand = ScanCommand(controller=self,
-                                        wavelegths=get_scanning_wavelengths(wl_step),
+                                        wavelengths=get_scanning_wavelengths(wl_step),
                                         positions=positions)
         self._scanCommandThread = threading.Thread(target=self._scanCommand.execute)
         self._scanCommandThread.start()
 
+    def start_fast_scan(self, wl_step=None, amplitude_x=None, amplitude_y=None):
+        self._check_ready()
+        if self._scanCommandThread:
+            if self._scanCommandThread.is_alive():
+                raise AlreadyScanningException
+            self._scanCommandThread = None
+        self._scanCommand = FastScanCommand(controller=self,
+                                            wavelengths=get_scanning_wavelengths(wl_step),
+                                            amplitude_x=amplitude_x,
+                                            amplitude_y=amplitude_y)
+        self._scanCommandThread = threading.Thread(target=self._scanCommand.execute)
+        self._scanCommandThread.start()
+
     def is_scan_command_running(self):
         if not self._scanCommandThread:
             return False
